searchData={"items":[{"type":"module","title":"Orange","doc":"Terminal UI framework for Elixir.\n\nA framework empowering developers to build complex terminal UI applications. Orange provides a declarative way to build UI components and manage terminal events. Orange offers stateful components which can encapsulate local state and logics, allows you to split your application into discrete sub-components. For example, here is a panel displays a random numbers every 1 second:\n    defmodule Panel do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: 0, events_subscription: true}\n\n      @impl true\n      def after_mount(_state, _attrs, update) do\n        spawn_link(fn -> random_number(update) end)\n      end\n\n      defp random_number(update) do\n        number = :rand.uniform(100)\n        update.(number)\n        Process.sleep(1000)\n        random_number(update)\n      end\n\n      @impl true\n      def handle_event(event, state, _attrs) do\n        case event do\n          %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n            Orange.stop()\n            state\n\n          _ ->\n            state\n        end\n      end\n\n      @impl true\n      def render(state, _attrs, _update) do\n        rect style: [width: 25, height: 10, border: true] do\n          rect do\n            \"Random number: #{state}\"\n          end\n        end\n      end\n    end\n\n    # Start the application. To quit, press 'q'.\n    Orange.start(Panel)\n\n![rendered result](assets/random-panel-example.gif)","ref":"Orange.html"},{"type":"function","title":"Orange.focus/1","doc":"Focus a component by component_id.\n\nThere can be only one focused component at a time. If a component is focused, it will receive all the events and prevent other components from receiving them. This is useful when you want a component to intercept all terminal events, for example, an input.","ref":"Orange.html#focus/1"},{"type":"function","title":"Orange.start/1","doc":"Start the runtime and render the UI root.\n\nThe root component must be a custom component.","ref":"Orange.html#start/1"},{"type":"function","title":"Orange.stop/0","doc":"Stop the runtime","ref":"Orange.html#stop/0"},{"type":"function","title":"Orange.subscribe/1","doc":"Subscribe to the event manager.\n\nSubscribed components will receive terminal events. To unsubscribe, use `unsubscribe/1`.","ref":"Orange.html#subscribe/1"},{"type":"function","title":"Examples - Orange.subscribe/1","doc":"defmodule Example do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: nil}\n\n      @impl true\n      def after_mount(_state, _attrs, _update) do\n        Orange.subscribe(:root)\n      end\n\n      @impl true\n      def render(_state, _attrs, _update) do\n        rect do\n          \"Hello\"\n        end\n      end\n    end\n\n    Orange.start({Example, id: :root})","ref":"Orange.html#subscribe/1-examples"},{"type":"function","title":"Orange.unfocus/1","doc":"Unfocus a component by component_id.","ref":"Orange.html#unfocus/1"},{"type":"function","title":"Orange.unsubscribe/1","doc":"Unsubscribe to the event manager.","ref":"Orange.html#unsubscribe/1"},{"type":"module","title":"Orange.Macro","doc":"Macros for creating Orange primitive components\n\nCurrently, Orange supports only one primitive component: `rect` (rectangle). Primitive components are\nthe building blocks of Orange applications. They can be used to construct more complex components.\n\nFor layout management, Orange leverages the [taffy](https://docs.rs/taffy/latest/taffy/) Rust crate.\nIt supports two layouts in the CSS specifications:\n\n  * `flex`: Flexible box layout\n  * `grid`: Grid layout system","ref":"Orange.Macro.html"},{"type":"module","title":"Examples - Orange.Macro","doc":"Macro provides an ergonomic way to create Orange components. For example, the following code:\n\n    rect style: [width: 12, border: true, flex_direction: :row] do\n      rect style: [color: :red] do\n        \"Hello\"\n      end\n\n      rect do\n        \"World\"\n      end\n    end\n\nwill produce the following struct:\n\n    %Orange.Rect{\n      children: [\n        %Orange.Rect{children: [\"Hello\"], attributes: [style: [color: :red]]},\n        %Orange.Rect{children: [\"World\"], attributes: []}\n      ],\n      attributes: [style: [width: 12, border: true, flex_direction: :row]]\n    }","ref":"Orange.Macro.html#module-examples"},{"type":"module","title":"Components children - Orange.Macro","doc":"Orange components can have children, provided by the do block. The syntax took inspiration from HTML:\n\n    rect do\n      rect do\n        \"Hello\"\n      end\n\n      rect do\n        \"World\"\n      end\n    end\n\nThe children can also be a list of components. This is useful when you want to render a collection of components. For example:\n\n    rect do\n      [\n        rect do\n          \"Hello\"\n        end,\n\n        rect do\n          \"World\"\n        end\n      ]\n    end\n\nNormal Elixir expression works inside the children block. For example, here's how to conditionally render components:\n\n    rect do\n      if :rand.uniform() > 0.5 do\n        \"Hello\"\n      else\n        \"World\"\n      end\n    end","ref":"Orange.Macro.html#module-components-children"},{"type":"module","title":"Styling - Orange.Macro","doc":"#","ref":"Orange.Macro.html#module-styling"},{"type":"module","title":"Display - Orange.Macro","doc":"The `:display` property controls how children are laid out. Supported values are:\n\n  * `:flex` (default) - Flexible box layout\n  * `:grid` - Grid layout\n\n##","ref":"Orange.Macro.html#module-display"},{"type":"module","title":"Flex Layout - Orange.Macro","doc":"When `display: :flex`, the following properties control the layout:\n\n  * `:flex_direction` - Direction of the flex container. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-direction) for more info. Supported values:\n\n    * `:row` (default) - Main axis is horizontal. Cross axis is vertical.\n\n    * `:column` - Main axis is vertical. Cross axis is horizontal.\n\n  * `:flex_grow` - How much the component grows relative to siblings when there is extra space. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-grow) for more info.\n\n  * `:flex_shrink` - How much the component shrinks relative to siblings when space is limited. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/flex-shrink) for more info.\n\n  * `:justify_content` - Alignment along the main axis. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/justify-content) for more info. Supported values:\n\n    * `:start` (default) - Pack items at the start\n\n    * `:end` - Pack items at the end\n\n    * `:center` - Center items\n\n    * `:space_between` - Evenly space items with first/last at edges\n\n    * `:space_around` - Evenly space items with equal space around\n\n    * `:space_evenly` - Evenly space items with equal space between\n\n    * `:stretch` - Stretch items to fill container\n\n  * `:align_items` - Alignment along the cross axis. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/align-items) for more info. Supported values:\n\n    * `:start` (default) - Align items at the start\n\n    * `:end` - Align items at the end\n\n    * `:center` - Center items\n\n    * `:space_between` - Evenly space items with first/last at edges\n\n    * `:space_around` - Evenly space items with equal space around\n\n    * `:space_evenly` - Evenly space items with equal space between\n\n    * `:stretch` - Stretch items to fill container\n\n##","ref":"Orange.Macro.html#module-flex-layout"},{"type":"module","title":"Grid Layout - Orange.Macro","doc":"When `display: :grid`, the following properties control the layout:\n\n  * `:grid_template_rows` - defines the rows in the grid. Takes a list of track sizes. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template-rows) for more info.\n  * `:grid_template_columns` - defines the columns in the grid. Takes a list of track sizes. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/grid-template-columns) for more info.\n\nTrack sizes can be:\n\n  * An integer - fixed number of cells\n\n  * A percentage string - percentage of container size (e.g. \"50%\")\n\n  * `{:fr, n}` - takes up n fraction of remaining space\n\n  * `:auto` - sized based on content\n\n  * `{:repeat, count, size}` - repeats the size specification count times\n\nChild items can be positioned in the grid using:\n\n  * `:grid_row` - specifies grid row placement. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/grid-row) for more info.\n\n  * `:grid_column` - specifies grid column placement. See [MDN docs](https://developer.mozilla.org/en-US/docs/Web/CSS/grid-column) for more info.\n\nGrid placement values can be:\n\n  * An integer - places at specific grid line\n\n  * `{:span, n}` - spans n tracks\n\n  * `:auto` - automatic placement\n\n  * `{start, end}` - explicit start/end placement where start/end is a grid placement (e.g. {2, {:span, 3}} means\n  start at row/column index 2 and ends at index 5)\n\n    ```\n    rect style: [\n      display: :grid,\n      grid_template_columns: [100, {:fr, 1}, \"50%\"],\n      grid_template_rows: [\"33%\", {:repeat, 2, {:fr, 1}}]\n    ] do\n      rect style: [grid_column: {1, 3}] do\n        \"Header\"\n      end\n      rect style: [grid_row: {:span, 2}] do\n        \"Sidebar\"\n      end\n    end\n    ```\n\n#","ref":"Orange.Macro.html#module-grid-layout"},{"type":"module","title":"Sizing - Orange.Macro","doc":"* `:width` - the width of the component\n\n  * `:height` - the height of the component\n\nThe values for `width` and `height` can be:\n\n  * An integer - the number of cells in the terminal. For example: `style: [width: 10]`\n\n  * A percentage string - the size is equal to the given percentage of the parent size. For example: `style: [width: \"50%\"]`\n\n#","ref":"Orange.Macro.html#module-sizing"},{"type":"module","title":"Padding - Orange.Macro","doc":"Padding for the render box's inner content. The values for `padding` can be:\n\n  * One integer - padding for all sides. For example: `style: [padding: 1]` means `[padding_top: 1, padding_bottom: 1, padding_left: 1, padding_right: 1]`\n\n  * Two integers tuple - padding vertical and padding horizontal. For example: `style: [padding: {1, 2}]` means `[padding_top: 1, padding_bottom: 1, padding_left: 2, padding_right: 2]`\n\n  * Four integers tuple - padding top, right, bottom, left respectively. For example: `style: [padding: {1, 2, 3, 4}]` means `[padding_top: 1, padding_bottom: 3, padding_left: 4, padding_right: 2]`\n\n#","ref":"Orange.Macro.html#module-padding"},{"type":"module","title":"Margin - Orange.Macro","doc":"Margin creates space around the render box. The values for `margin` follow the same format as padding:\n\n  * One integer - margin for all sides. For example: `style: [margin: 1]` means `[margin_top: 1, margin_bottom: 1, margin_left: 1, margin_right: 1]`\n\n  * Two integers tuple - margin vertical and margin horizontal. For example: `style: [margin: {1, 2}]` means `[margin_top: 1, margin_bottom: 1, margin_left: 2, margin_right: 2]`\n\n  * Four integers tuple - margin top, right, bottom, left respectively. For example: `style: [margin: {1, 2, 3, 4}]` means `[margin_top: 1, margin_bottom: 3, margin_left: 4, margin_right: 2]`\n\n#","ref":"Orange.Macro.html#module-margin"},{"type":"module","title":"Border - Orange.Macro","doc":"* `:border` - whether to render a border around the rect. Defaults to `false`\n\n  * `:border_top` - whether to render a top border. Defaults to `true` if `border` is `true`\n\n  * `:border_bottom` - whether to render a bottom border. Defaults to `true` if `border` is `true`\n\n  * `:border_left` - whether to render a left border. Defaults to `true` if `border` is `true`\n\n  * `:border_right` - whether to render a right border. Defaults to `true` if `border` is `true`\n\n  * `:border_color` - the color of the border. See [Color](#module-color) section for supported colors\n\n#","ref":"Orange.Macro.html#module-border"},{"type":"module","title":"Text modifiers - Orange.Macro","doc":"An array of modifiers which adjust the text style. Supported modifiers are:\n\n  * `:bold`\n  * `:dim`\n  * `:italic`\n  * `:underline`\n  * `:strikethrough`\n\n#","ref":"Orange.Macro.html#module-text-modifiers"},{"type":"module","title":"Color - Orange.Macro","doc":"* `:color` - the color of the component text. The color value can be inherited from the parent component. If the color value is not specified, the component will inherit the color from the parent component\n\n  * `:background_color` - the color of the component background\n\nThe values for `:color` and `:background_color` is a single atom representing the color. Supported colors are:\n\n  * `:white`\n  * `:black`\n  * `:grey`\n  * `:dark_grey`\n  * `:red`\n  * `:dark_red`\n  * `:green`\n  * `:dark_green`\n  * `:yellow`\n  * `:dark_yellow`\n  * `:blue`\n  * `:dark_blue`\n  * `:magenta`\n  * `:dark_magenta`\n  * `:cyan`\n  * `:dark_cyan`\n\n#","ref":"Orange.Macro.html#module-color"},{"type":"module","title":"Line wrap - Orange.Macro","doc":"By default, the text will wrap to the next line if it exceeds the width of the component. To disable this behavior, set the `:line_wrap` attribute to `false`","ref":"Orange.Macro.html#module-line-wrap"},{"type":"module","title":"Position - Orange.Macro","doc":"Rect elements support positioning. Supported values are:\n\n  * `{:fixed, top, right, bottom, left}` - the rect will be fixed at the given position. The values are offset to the respective edge of the screen. For example:\n\n    ```\n    rect position: {:fixed, 1, 2, 1, 2} do\n      \"Fixed position\"\n    end\n    ```\n\n    will render:\n\n    ![rendered result](assets/fixed-position-example.png)","ref":"Orange.Macro.html#module-position"},{"type":"macro","title":"Orange.Macro.rect/2","doc":"Generates a `Orange.Rect` struct","ref":"Orange.Macro.html#rect/2"},{"type":"macro","title":"Options - Orange.Macro.rect/2","doc":"* `:style` - style attributes for the rect. Supported keys are:\n\n    * `:width` - see [Sizing](#module-sizing) section\n\n    * `:height` - see [Sizing](#module-sizing) section\n\n    * `:border` - see [Border](#module-border) section\n\n    * `:padding` - see [Padding](#module-padding) section\n\n    * `:color` - see [Color](#module-color) section\n\n    * `:background_color` - see [Color](#module-color) section\n\n    * `:display` - see [Display][#module-display] section\n\n    * `:flex_direction` - available for `:flex` display. See [Flex Layout](#module-flex-layout) section\n\n    * `:justify_content` - available for `:flex` display. See [Flex Layout](#module-flex-layout) section\n\n    * `:align_items` - available for `:flex` display. See [Flex Layout](#module-flex-layout) section\n\n    * `:flex_grow` - available for `:flex` display. See [Flex Layout](#module-flex-layout) section\n\n    * `:flex_shrink` - available for `:flex` display. See [Flex Layout](#module-flex-layout) section\n\n    * `:grid_template_rows` - available for `:grid` display. See [Grid Layout](#module-grid-layout) section\n\n    * `:grid_template_columns` - available for `:grid` display. See [Grid Layout](#module-grid-layout) section\n\n    * `:grid_row` - available for `:grid` display. See [Grid Layout](#module-grid-layout) section\n\n    * `:grid_column` - available for `:grid` display. See [Grid Layout](#module-grid-layout) section\n\n  * `:title` - the title of the rect. If specified, it implies `border` is `true`. The title can be a string or a map with supported keys are:\n\n    * `:text` - the title text. This field is required\n\n    * `:color` - the title color. This field is optional\n\n    * `:text_modifiers` - the title text modifiers. See [Text modifiers](#module-text-modifiers) section. This field is optional\n\n    * `:offset` - an integer specifies the title offset from the left edge. This field is optional and defaults to 0\n\n  * `:scroll_x` - the horizontal scroll offset\n\n  * `:scroll_y` - the vertical scroll offset\n\n  * `:position` - the position of the rect. See [Position](#module-position) section","ref":"Orange.Macro.html#rect/2-options"},{"type":"macro","title":"Examples - Orange.Macro.rect/2","doc":"iex> import Orange.Macro\n    iex> rect style: [width: 5, border: true, flex_direction: :row] do\n    ...>   \"foo\"\n    ...>\n    ...>   rect do\n    ...>     \"bar\"\n    ...>   end\n    ...> end\n    %Orange.Rect{\n      children: [\n        %Orange.Rect{children: [\"foo\"], attributes: []},\n        %Orange.Rect{children: [\"bar\"], attributes: []}\n      ],\n      attributes: [style: [width: 5, border: true, flex_direction: :row]]\n    }","ref":"Orange.Macro.html#rect/2-examples"},{"type":"behaviour","title":"Orange.Component","doc":"A behaviour module for implementing custom components.\n\nA custom component is a self-contained UI component that can be reused across the application. Custom components are composed of other custom or primitive components.\n\nCustom components can have internal state and receive terminal events to update their state. They also have hooks, which will be trigger at specific points in the component lifecycle, for example, `after_mount/1` and `after_unmount/1`","ref":"Orange.Component.html"},{"type":"behaviour","title":"Examples - Orange.Component","doc":"In order to implement a custom component, you need to define a module that implements the `Orange.Component` behaviour. As an example, we will implement a Counter component. We can use up and down button to increase or decrease the counter value.\n\n    defmodule Counter do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: 0, events_subscription: true}\n\n      @impl true\n      def handle_event(event, state, _attrs) do\n        case event do\n          %Orange.Terminal.KeyEvent{code: :up} ->\n            state + 1\n\n          %Orange.Terminal.KeyEvent{code: :down} ->\n            state - 1\n\n          _ ->\n            state\n        end\n      end\n\n      @impl true\n      def after_mount(state, _attrs, update) do\n        # Set counter value to 10 after 5 seconds\n        spawn(fn ->\n          Process.sleep(5000)\n          update.(10)\n        end)\n      end\n\n      @impl true\n      def render(state, attrs, _update) do\n        rect style: [width: \"100%\", height: \"1fr\", border: attrs[:highlight]] do\n          rect do\n            \"Counter: #{state}\"\n          end\n        end\n      end\n    end\n\n\nThis component can be used as children for other components like this:\n\n    defmodule App do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: nil, events_subscription: true}\n\n      @impl true\n      def handle_event(event, state, _attrs) do\n        case event do\n          %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n            Orange.stop()\n            state\n\n          _ ->\n            state\n        end\n      end\n\n      def render(_state, _attrs, _update) do\n        rect style: [width: 20, height: 20, border: true] do\n          # No attributes\n          Counter\n\n          # Or with attributes\n          {Counter, id: :counter, highlight: true}\n        end\n      end\n    end\n\n![Rendered result](assets/component-example.gif)","ref":"Orange.Component.html#module-examples"},{"type":"behaviour","title":"Update callback - Orange.Component","doc":"There are two ways to update the component state:\n\n  1. By returning a new state from the `handle_event/3` callback\n\n  2. By using the update callback\n\nThe update callback will be passed to the `render/3` and lifecycle hooks. The update callback can be called with a new state or a function that receives the current state and returns the new state.\n\n    def render(state, attrs, update_callback)\n      update_callback.(%{state | counter: state.counter + 1})\n      update_callback.(fn state -> %{state | counter: state.counter - 1} end)\n    end","ref":"Orange.Component.html#module-update-callback"},{"type":"callback","title":"Orange.Component.after_mount/3","doc":"Lifecycle hook that is called after the component is mounted (first time render to the terminal).","ref":"Orange.Component.html#c:after_mount/3"},{"type":"callback","title":"Orange.Component.after_unmount/3","doc":"Lifecycle hook that is called after the component is unmounted (remove from the terminal).","ref":"Orange.Component.html#c:after_unmount/3"},{"type":"callback","title":"Orange.Component.handle_event/3","doc":"Receives the event, current state, and attributes and returns the new state.","ref":"Orange.Component.html#c:handle_event/3"},{"type":"callback","title":"Orange.Component.init/1","doc":"Receives the attributes map from parent component and returns the component configuration.\n\nConfiguration keys are:\n\n  * `state` - The initial state of the component\n\n  * `events_subscription` - whether the component should subscribe to terminal events","ref":"Orange.Component.html#c:init/1"},{"type":"callback","title":"Orange.Component.render/3","doc":"Receives the current state, attributes and [update callback](#module-update-callback) and returns the rendered UI element.\n\n> ##","ref":"Orange.Component.html#c:render/3"},{"type":"callback","title":"Info {: .info} - Orange.Component.render/3","doc":">\n> This function must either return a single component or nil.","ref":"Orange.Component.html#c:render/3-info-info"},{"type":"type","title":"Orange.Component.event/0","doc":"","ref":"Orange.Component.html#t:event/0"},{"type":"type","title":"Orange.Component.state/0","doc":"","ref":"Orange.Component.html#t:state/0"},{"type":"type","title":"Orange.Component.ui_element/0","doc":"","ref":"Orange.Component.html#t:ui_element/0"},{"type":"type","title":"Orange.Component.update_callback/0","doc":"","ref":"Orange.Component.html#t:update_callback/0"},{"type":"module","title":"Orange.Component.Input","doc":"An uncontrolled input component.","ref":"Orange.Component.Input.html"},{"type":"module","title":"Attributes - Orange.Component.Input","doc":"- `:on_submit` - A callback triggered when input is submitted. This attribute is required.\n  - `:submit_key` - The keyboard key that will trigger submission. This attribute is optional and defaults to :enter. See `Orange.Terminal.KeyEvent` for supported values.\n  - `:on_exit` - A callback triggered when input is exited. This attribute is optional.\n  - `:exit_key` - The keyboard key that will unfocus the input. This attribute is optional and defaults to :esc. See `Orange.Terminal.KeyEvent` for supported values.\n  > ##","ref":"Orange.Component.Input.html#module-attributes"},{"type":"module","title":"Info {: .info} - Orange.Component.Input","doc":">\n  > The `:submit_key` and `:exit_key` can not be `:backspace` as it is reserved for deleting characters.\n\n  - `:auto_focus` - Whether to focus automatically after mount. This attribute is optional. If true, the `:id` attribute is required and the input will unfocus after submission.\n  - `:prefix` - The input prefix string. This attribute is optional.\n  - `:style` - The component style. This attribute is optional.","ref":"Orange.Component.Input.html#module-info-info"},{"type":"module","title":"Examples - Orange.Component.Input","doc":"defmodule Example do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: %{search_value: \"\"}}\n\n      @impl true\n      def render(state, _attrs, update) do\n        rect style: [width: 50, height: 20] do\n          {\n            Orange.Component.Input,\n            id: :input,\n            on_submit: fn value -> update.(%{state | search_value: value}) end,\n            submit_key: {:char, \"x\"},\n            prefix: \"Enter search value:\",\n            auto_focus: true\n          }\n\n          rect do\n            \"Search value: #{state.search_value}\"\n          end\n        end\n      end\n    end\n\n  ![rendered result](assets/input-example.gif)","ref":"Orange.Component.Input.html#module-examples"},{"type":"module","title":"Orange.Component.Modal","doc":"An modal/dialog component which renders an overlay in the middle of the viewport.","ref":"Orange.Component.Modal.html"},{"type":"module","title":"Attributes - Orange.Component.Modal","doc":"- `:open` - Whether the modal is open or not. This attribute is required.\n  - `:offset_x` - The offset from the left and right edge of the screen. This attribute is required.\n  - `:offset_y` - The offset from the top and bottom edge of the screen. This attribute is required.\n  > ##","ref":"Orange.Component.Modal.html#module-attributes"},{"type":"module","title":"Info {: .info} - Orange.Component.Modal","doc":">\n  > When the offset_x/y is too big for the terminal size, the modal will not be rendered.\n\n  - `:children` - A list of elements used as content of the modal. This attribute is optional.\n  - `:title` - The title of the modal. See `Orange.Macro.rect/2` for supported values. This attribute is optional.\n  - `:style` - The modal style. See `Orange.Macro.rect/2` for supported values. This attribute is optional.","ref":"Orange.Component.Modal.html#module-info-info"},{"type":"module","title":"Examples - Orange.Component.Modal","doc":"defmodule Example do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: %{search_value: \"\"}}\n\n      @impl true\n      def render(state, _attrs, update) do\n        modal_content =\n          rect do\n            \"foo\"\n            \"bar\"\n          end\n\n        rect do\n          rect do\n            \"Displaying modal...\"\n          end\n\n          {\n            Orange.Component.Modal,\n            offset_x: 8,\n            offset_y: 4,\n            children: [modal_content],\n            open: true\n          }\n        end\n      end\n    end\n\n  ![rendered result](assets/modal-example.png)","ref":"Orange.Component.Modal.html#module-examples"},{"type":"module","title":"Orange.Component.VerticalScrollableRect","doc":"A component that renders a vertical scrollable rectangle. When the container is overflown, a scrollbar is rendered.","ref":"Orange.Component.VerticalScrollableRect.html"},{"type":"module","title":"Attributes - Orange.Component.VerticalScrollableRect","doc":"- `:height` - The height of the container rect. This attribute is required.\n  - `:content_height` - The height of the inner content. This attribute is required.\n  - `:scroll_offset` - The vertical scroll offset. This attribute is required.\n  - `:children` - A list of elements rendered inside the container rect. This attribute is required.\n  - `:title` - The title of the scrolling box. This attribute is optional.","ref":"Orange.Component.VerticalScrollableRect.html#module-attributes"},{"type":"module","title":"Examples - Orange.Component.VerticalScrollableRect","doc":"defmodule Example do\n      @behaviour Orange.Component\n\n      import Orange.Macro\n\n      @impl true\n      def init(_attrs), do: %{state: nil}\n\n      @impl true\n      def render(_state, attrs, _update) do\n        total_items = 30\n\n        rect style: [width: 20, height: 20, border: attrs[:highlight]] do\n          {\n            Orange.Component.VerticalScrollableRect,\n            height: 20,\n            content_height: total_items,\n            scroll_offset: 5,\n            children: for(i <- 1..total_items, do: \"Item #{i}\")\n          }\n        end\n      end\n    end\n\n  ![rendered result](assets/vertical-scrollable-rect-example.png)\n\n  For more examples, see `examples/scrollable_rect.exs`.","ref":"Orange.Component.VerticalScrollableRect.html#module-examples"},{"type":"module","title":"Orange.Terminal.KeyEvent","doc":"Terminal keyboard events.\n\nSee `key_code/0` below for supported keys.","ref":"Orange.Terminal.KeyEvent.html"},{"type":"type","title":"Orange.Terminal.KeyEvent.key_code/0","doc":"","ref":"Orange.Terminal.KeyEvent.html#t:key_code/0"},{"type":"type","title":"Orange.Terminal.KeyEvent.key_modifier/0","doc":"","ref":"Orange.Terminal.KeyEvent.html#t:key_modifier/0"},{"type":"type","title":"Orange.Terminal.KeyEvent.t/0","doc":"","ref":"Orange.Terminal.KeyEvent.html#t:t/0"},{"type":"module","title":"Orange.Terminal.ResizeEvent","doc":"Terminal resize event.","ref":"Orange.Terminal.ResizeEvent.html"},{"type":"type","title":"Orange.Terminal.ResizeEvent.t/0","doc":"","ref":"Orange.Terminal.ResizeEvent.html#t:t/0"},{"type":"extras","title":"Handle terminal events","doc":"# Handle terminal events\n\nCustom components can have local state. To make them becomes interactive, they need to react to terminal events. There are two ways to achieve that:","ref":"events-subscription.html"},{"type":"extras","title":"Init configuration - Handle terminal events","doc":"In component's init callback, we can return a configuration with `events_subscription: true` to subscribe events for the component.\n\n```elixir\ndefmodule Example do\n  @behaviour Orange.Component\n\n  import Orange.Macro\n\n  @impl true\n  def init(_attrs), do: %{state: nil, events_subscription: true}\n\n  @impl true\n  def handle_event(event, state, _attrs) do\n    case event do\n      %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n        Orange.stop()\n        state\n\n      %Orange.Terminal.KeyEvent{code: {:char, char}} ->\n        IO.puts(\"Key pressed: #{char}\")\n        state\n\n      _ ->\n        state\n    end\n  end\n\n  @impl true\n  def render(state, _attrs, _update) do\n    rect style: [width: 20, height: 20] do\n      \"Hello\"\n    end\n  end\nend\n\n# Start the application. To quit, press 'q'.\nOrange.start(Example)\n```\n\nYour component can handle events with the `handle_event/3` callback and returns the updated state.","ref":"events-subscription.html#init-configuration"},{"type":"extras","title":"Orange.subscribe/1 - Handle terminal events","doc":"If you want to manually subscribe, use `Orange.subscribe/1`.\n\n```elixir\ndefmodule Example do\n  @behaviour Orange.Component\n\n  import Orange.Macro\n\n  @impl true\n  def init(_attrs), do: %{state: nil}\n\n  @impl true\n  def after_mount(_state, _attrs, _update) do\n    Orange.subscribe(:root)\n  end\n\n  @impl true\n  def handle_event(event, state, _attrs) do\n    case event do\n      %Orange.Terminal.KeyEvent{code: {:char, \"q\"}} ->\n        Orange.stop()\n        state\n\n      _ ->\n        state\n    end\n  end\n\n  @impl true\n  def render(state, _attrs, _update) do\n    rect style: [width: 20, height: 20] do\n      \"Hello\"\n    end\n  end\nend\n\n# Start the application. To quit, press 'q'.\nOrange.start({Example, id: :root})\n```","ref":"events-subscription.html#orange-subscribe-1"},{"type":"extras","title":"Remove events subscription - Handle terminal events","doc":"To unsubcribe for events, use `Orange.unsubcribe/1`.","ref":"events-subscription.html#remove-events-subscription"},{"type":"extras","title":"Component focus mode - Handle terminal events","doc":"There are certain cases that you only want one component to receive events and prevent other components. Look at the above example, an input component should receive all the events and prevent users from quitting the application when hitting \"q\". To do that, we can set a component as focused with `Orange.focus/1`. Only the focused component can receive terminal events. To remove the focus status, use `Orange.unfocus/1`.\n\n# Supported events\n\n- Keyboard events: see `Orange.Terminal.KeyEvent` for more details\n- Terminal resize event: see `Orange.Terminal.ResizeEvent` for more details","ref":"events-subscription.html#component-focus-mode"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}